@@ -5,9 +5,7 @@
 
 import os
 import sys
-
-from django.core.handlers.wsgi import WSGIHandler
-from django.core.servers.basehttp import AdminMediaHandler, WSGIServerException
+import traceback
 
 from gunicorn.config import Config
 from gunicorn.app.base import Application
@@ -15,8 +13,9 @@ from gunicorn.app.base import Application
 class DjangoApplication(Application):
     
     def init(self, parser, opts, args):
+        from django.conf import ENVIRONMENT_VARIABLE
+        self.settings_modname = None
         self.project_path = os.getcwd()
-    
         if args:
             settings_path = os.path.abspath(os.path.normpath(args[0]))
             if not os.path.exists(settings_path):
@@ -24,49 +23,86 @@ class DjangoApplication(Application):
             else:
                 self.project_path = os.path.dirname(settings_path)
         else:
-             settings_path = os.path.join(self.project_path, "settings.py")
-             if not os.path.exists(settings_path):
-                 self.no_settings(settings_path)
+            try:
+                self.settings_modname = os.environ[ENVIRONMENT_VARIABLE]
+            except KeyError:
+                settings_path = os.path.join(self.project_path, "settings.py")
+                if not os.path.exists(settings_path):
+                    return self.no_settings(settings_path)
 
-        project_name = os.path.split(self.project_path)[-1]
-        settings_name, ext  = os.path.splitext(os.path.basename(settings_path))
-        self.settings_modname = "%s.%s" % (project_name, settings_name)
+        if not self.settings_modname:
+            project_name = os.path.split(self.project_path)[-1]
+            settings_name, ext  = os.path.splitext(
+                    os.path.basename(settings_path))
+            self.settings_modname = "%s.%s" % (project_name, settings_name)
+            os.environ[ENVIRONMENT_VARIABLE] = self.settings_modname
+        
         self.cfg.set("default_proc_name", self.settings_modname)
 
+        # add the project path to sys.path
         sys.path.insert(0, self.project_path)
         sys.path.append(os.path.join(self.project_path, os.pardir))
 
-    def no_settings(self, path):
-        error = "Settings file '%s' not found in current folder.\n" % path
+        # setup envoron
+        self.setup_environ() 
+
+    def setup_environ(self):
+        from django.core.management import setup_environ
+        try:
+            parts = self.settings_modname.split(".")
+            settings_mod = __import__(self.settings_modname)
+            if len(parts) > 1:
+                settings_mod = __import__(parts[0])
+                path = os.path.dirname(os.path.abspath(
+                            os.path.normpath(settings_mod.__file__)))
+                sys.path.append(path)
+                for part in parts[1:]: 
+                    settings_mod = getattr(settings_mod, part)
+                setup_environ(settings_mod)
+        except ImportError:
+            return self.no_settings(self.settings_modname, import_error=True)
+
+    def no_settings(self, path, import_error=False):
+        if import_error:
+            error = "Error: Can't find the settings in your PYTHONPATH"
+        else:
+            error = "Settings file '%s' not found in current folder.\n" % path
         sys.stderr.write(error)
         sys.stderr.flush()
         sys.exit(1)
         
     def load(self):
-        os.environ['DJANGO_SETTINGS_MODULE'] = self.settings_modname
+        from django.conf import ENVIRONMENT_VARIABLE
+        from django.core.handlers.wsgi import WSGIHandler
+        os.environ[ENVIRONMENT_VARIABLE] = self.settings_modname
         return WSGIHandler()
 
 class DjangoApplicationCommand(Application):
     
     def __init__(self, options, admin_media_path):
-        self.cfg = Config()
+        self.usage = None
+        self.cfg = None
+        self.config_file = options.get("config") or ""
+        self.options = options
+        self.admin_media_path = admin_media_path
         self.callable = None
+        self.do_load_config()
 
-        # Load up the config file if its found.
-
-        config_file = options.get("config") or ""
-        if config_file and os.path.exists(config_file):
+    def load_config(self):
+        self.cfg = Config()
+        
+        if self.config_file and os.path.exists(self.config_file):
             cfg = {
                 "__builtins__": __builtins__,
                 "__name__": "__config__",
-                "__file__": config_file,
+                "__file__": self.config_file,
                 "__doc__": None,
                 "__package__": None
             }
             try:
-                execfile(config_file, cfg, cfg)
-            except Exception, e:
-                print "Failed to read config file: %s" % config_file
+                execfile(self.config_file, cfg, cfg)
+            except Exception:
+                print "Failed to read config file: %s" % self.config_file
                 traceback.print_exc()
                 sys.exit(1)
         
@@ -80,13 +116,13 @@ class DjangoApplicationCommand(Application):
                     sys.stderr.write("Invalid value for %s: %s\n\n" % (k, v))
                     raise
         
-        for k, v in list(options.items()):
+        for k, v in list(self.options.items()):
             if k.lower() in self.cfg.settings and v is not None:
                 self.cfg.set(k.lower(), v)
         
-        self.admin_media_path = admin_media_path
-        
     def load(self):
+        from django.core.servers.basehttp import AdminMediaHandler, WSGIServerException
+        from django.core.handlers.wsgi import WSGIHandler
         try:
             return  AdminMediaHandler(WSGIHandler(), self.admin_media_path)
         except WSGIServerException, e:
