@@ -18,7 +18,8 @@ from gunicorn.errors import HaltServer
 from gunicorn.pidfile import Pidfile
 from gunicorn.sock import create_socket
 from gunicorn import util
-from gunicorn import __version__
+
+from gunicorn import __version__, SERVER_SOFTWARE
 
 class Arbiter(object):
     """
@@ -52,7 +53,9 @@ class Arbiter(object):
     def __init__(self, app):
         self.log = logging.getLogger(__name__)
         self.log.info("Starting gunicorn %s" % __version__)
-        
+       
+        os.environ["SERVER_SOFTWARE"] = SERVER_SOFTWARE
+
         self.setup(app)
         
         self.pidfile = None
@@ -115,7 +118,8 @@ class Arbiter(object):
             self.pidfile = Pidfile(self.cfg.pidfile)
             self.pidfile.create(self.pid)
         self.log.debug("Arbiter booted")
-        self.log.info("Listening at: %s" % self.LISTENER)
+        self.log.info("Listening at: %s (%s)" % (self.LISTENER,
+            self.pid))
         self.cfg.when_ready(self)
     
     def init_signals(self):
@@ -142,6 +146,7 @@ class Arbiter(object):
         "Main master loop."
         self.start()
         util._setproctitle("master [%s]" % self.proc_name)
+        
         self.manage_workers()
         while True:
             try:
@@ -247,7 +252,8 @@ class Arbiter(object):
     def handle_winch(self):
         "SIGWINCH handling"
         if os.getppid() == 1 or os.getpgrp() != os.getpid():
-            self.logger.info("graceful stop of workers")
+            self.log.info("graceful stop of workers")
+            self.num_workers = 0
             self.kill_workers(signal.SIGQUIT)
         else:
             self.log.info("SIGWINCH ignored. Not daemonized")
@@ -358,7 +364,7 @@ class Arbiter(object):
         util._setproctitle("master [%s]" % self.proc_name)
         
         # manage workers
-        self.manage_workers()
+        self.manage_workers() 
         
     def murder_workers(self):
         """\
@@ -374,7 +380,7 @@ class Arbiter(object):
 
             self.log.critical("WORKER TIMEOUT (pid:%s)" % pid)
             self.kill_worker(pid, signal.SIGKILL)
-    
+        
     def reap_workers(self):
         """\
         Reap workers to avoid zombie processes
@@ -447,7 +453,6 @@ class Arbiter(object):
             try:
                 worker.tmp.close()
                 self.cfg.worker_exit(self, worker)
-                os.unlink(worker.tmpname)
             except:
                 pass
 
@@ -485,7 +490,6 @@ class Arbiter(object):
                     worker = self.WORKERS.pop(pid)
                     worker.tmp.close()
                     self.cfg.worker_exit(self, worker)
-                    os.unlink(worker.tmpname)
                     return
                 except (KeyError, OSError):
                     return
