@@ -14,21 +14,6 @@ import sys
 import textwrap
 import time
 
-try:#python 2.6, use subprocess
-    import subprocess
-    subprocess.Popen  # trigger ImportError early
-    closefds = os.name == 'posix'
-    
-    def popen3(cmd, mode='t', bufsize=0):
-        p = subprocess.Popen(cmd, shell=True, bufsize=bufsize,
-            stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, 
-            close_fds=closefds)
-        p.wait()
-        return (p.stdin, p.stdout, p.stderr)
-except ImportError:
-    subprocess = None
-    popen3 = os.popen3
-
 
 MAXFD = 1024
 if (hasattr(os, "devnull")):
@@ -60,7 +45,7 @@ hop_headers = set("""
     te trailers transfer-encoding upgrade
     server date
     """.split())
-             
+            
 try:
     from setproctitle import setproctitle
     def _setproctitle(title):
@@ -114,20 +99,39 @@ def chown(path, uid, gid):
         os.chown(path, uid, gid)
     except OverflowError:
         os.chown(path, uid, -ctypes.c_int(-gid).value)
+
+
+def is_ipv6(addr):
+    try:
+        socket.inet_pton(socket.AF_INET6, addr)
+    except socket.error: # not a valid address
+        return False
+    return True
         
-def parse_address(host, port=None, default_port=8000):
-    if host.startswith("unix:"):
-        return host.split("unix:")[1]
-        
-    if not port:
-        if ':' in host:
-            host, port = host.split(':', 1)
-            if not port.isdigit():
-                raise RuntimeError("%r is not a valid port number." % port)
-            port = int(port)
-        else:
-            port = default_port
-    return (host, int(port))
+def parse_address(netloc, default_port=8000):
+    if netloc.startswith("unix:"):
+        return netloc.split("unix:")[1]
+
+    # get host
+    if '[' in netloc and ']' in netloc:
+        host = netloc.split(']')[0][1:].lower()
+    elif ':' in netloc:
+        host = netloc.split(':')[0].lower()
+    elif netloc == "":
+        host = "0.0.0.0"
+    else:
+        host = netloc.lower()
+    
+    #get port
+    netloc = netloc.split(']')[-1]
+    if ":" in netloc:
+        port = netloc.split(':', 1)[1]
+        if not port.isdigit():
+            raise RuntimeError("%r is not a valid port number." % port)
+        port = int(port)
+    else:
+        port = default_port 
+    return (host, port)
     
 def get_maxfd():
     maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
@@ -174,27 +178,28 @@ def writelines(sock, lines, chunked=False):
     for line in list(lines):
         write(sock, line, chunked)
 
-def write_error(sock, msg):
+def write_error(sock, msg, status_int=500, 
+        reason="Internal Server Error"):
     html = textwrap.dedent("""\
     <html>
         <head>
-            <title>Internal Server Error</title>
+            <title>%(reason)s</title>
         </head>
         <body>
-            <h1>Internal Server Error</h1>
-            <h2>WSGI Error Report:</h2>
-            <pre>%s</pre>
+            <h1>%(reason)s</h1>
+            %(msg)s
         </body>
     </html>
-    """) % msg
+    """) % {"reason": reason, "msg": msg}
+
     http = textwrap.dedent("""\
-    HTTP/1.1 500 Internal Server Error\r
+    HTTP/1.1 %s %s\r
     Connection: close\r
     Content-Type: text/html\r
     Content-Length: %d\r
     \r
     %s
-    """) % (len(html), html)
+    """) % (str(status_int), reason, len(html), html)
     write_nonblock(sock, http)
 
 def normalize_name(name):
@@ -206,7 +211,16 @@ def import_app(module):
         module, obj = module, "application"
     else:
         module, obj = parts[0], parts[1]
-    __import__(module)
+
+    try:
+        __import__(module)
+    except ImportError:
+        if module.endswith(".py") and os.path.exists(module):
+            raise ImportError("Failed to find application, did "
+                "you mean '%s:%s'?" % (module.rsplit(".",1)[0], obj))
+        else:
+            raise
+
     mod = sys.modules[module]
     app = eval(obj, mod.__dict__)
     if app is None:
