@@ -7,7 +7,6 @@ from __future__ import with_statement
 
 import errno
 import socket
-import traceback
 
 import gunicorn.http as http
 import gunicorn.http.wsgi as wsgi
@@ -33,11 +32,9 @@ class AsyncWorker(base.Worker):
                     req = None
                     with self.timeout_ctx():
                         req = parser.next()
-
                     if not req:
                         break
                     self.handle_request(req, client, addr)
-                
             except StopIteration:
                 pass
         except socket.error, e:
@@ -50,22 +47,19 @@ class AsyncWorker(base.Worker):
                     self.log.debug("Ignoring EPIPE")
         except Exception, e:
             self.log.exception("General error processing request.")
-            try:            
-                # Last ditch attempt to notify the client of an error.
-                mesg = "HTTP/1.0 500 Internal Server Error\r\n\r\n"
-                util.write_nonblock(client, mesg)
-            except:
-                pass
-            return
+            self.handle_error(client, e)
         finally:
             util.close(client)
 
     def handle_request(self, req, sock, addr):
         try:
-            debug = self.cfg.debug or False
             self.cfg.pre_request(self, req)
-            resp, environ = wsgi.create(req, sock, addr, self.address,
-                    self.cfg)
+            resp, environ = wsgi.create(req, sock, addr, self.address, self.cfg)
+            self.nr += 1
+            if self.alive and self.nr >= self.max_requests:
+                self.log.info("Autorestarting worker after current request.")
+                resp.force_close()
+                self.alive = False
             respiter = self.wsgi(environ, resp.start_response)
             if respiter == ALREADY_HANDLED:
                 return False
@@ -76,13 +70,15 @@ class AsyncWorker(base.Worker):
                 respiter.close()
             if req.should_close():
                 raise StopIteration()
-            self.cfg.post_request(self, req)
         except StopIteration:
             raise
         except Exception, e:
             #Only send back traceback in HTTP in debug mode.
-            if not self.debug:
-                raise
-            util.write_error(sock, traceback.format_exc())
+            self.handle_error(sock, e)
             return False
+        finally:
+            try:
+                self.cfg.post_request(self, req)
+            except:
+                pass
         return True
