@@ -9,9 +9,15 @@ import os
 import random
 import signal
 import sys
-import tempfile
+import traceback
+
 
 from gunicorn import util
+from gunicorn.workers.workertmp import WorkerTmp
+
+from gunicorn.http.errors import InvalidHeader, InvalidHeaderName, \
+InvalidRequestLine, InvalidRequestMethod, InvalidHTTPVersion
+
 
 class Worker(object):
 
@@ -39,14 +45,10 @@ class Worker(object):
         self.nr = 0
         self.max_requests = cfg.max_requests or sys.maxint
         self.alive = True
-        self.spinner = 0
         self.log = logging.getLogger(__name__)
         self.debug = cfg.debug
         self.address = self.socket.getsockname()
-
-        self.fd, self.tmpname = tempfile.mkstemp(prefix="wgunicorn-")
-        util.chown(self.tmpname, cfg.uid, cfg.gid)
-        self.tmp = os.fdopen(self.fd, "r+b")
+        self.tmp = WorkerTmp(cfg) 
         
     def __str__(self):
         return "<Worker %s>" % self.pid
@@ -61,11 +63,7 @@ class Worker(object):
         once every ``self.timeout`` seconds. If you fail in accomplishing
         this task, the master process will murder your workers.
         """
-        self.spinner = (self.spinner+1) % 2
-        if getattr(os, 'fchmod', None):
-            os.fchmod(self.tmp.fileno(), self.spinner)
-        else:
-            os.chmod(self.tmpname, self.spinner)
+        self.tmp.notify()
 
     def run(self):
         """\
@@ -94,7 +92,7 @@ class Worker(object):
         
         # Prevent fd inherientence
         util.close_on_exec(self.socket)
-        util.close_on_exec(self.fd)
+        util.close_on_exec(self.tmp.fileno())
         self.init_signals()
         
         self.wsgi = self.app.wsgi()
@@ -117,6 +115,35 @@ class Worker(object):
         self.alive = False
         sys.exit(0)
 
+    def handle_error(self, client, exc):
+        if isinstance(exc, (InvalidRequestLine, InvalidRequestMethod,
+            InvalidHTTPVersion, InvalidHeader, InvalidHeaderName,)):
+            
+            if isinstance(exc, InvalidRequestLine):
+                mesg = "<p>Invalid Request Line '%s'</p>" % str(exc)
+            elif isinstance(exc, InvalidRequestMethod):
+                mesg = "<p>Invalid Method'%s'</p>" % str(exc)
+            elif isinstance(exc, InvalidHTTPVersion):
+                mesg = "<p>Invalid HTTP Version '%s'</p>" % str(exc)
+            elif isinstance(exc, (InvalidHeaderName, InvalidHeader,)):
+                mesg = "<p>Invalid Header'%s'</p>" % str(exc)
+            reason = "Bad Request"
+            status_int = 400
+        else:
+            mesg = reason = "Internal Server reason"
+            status_int = 500
+            
+        if self.debug:
+            tb =  traceback.format_exc()
+            mesg += "<h2>Traceback:</23><pre>%s</pre>" % tb
+
+        try:
+            util.write_error(client, mesg, status_int=status_int, 
+                    reason=reason)
+        except:
+            if self.debug:
+                self.log.warning("Unexpected error %s" % traceback.format_exc())
+        
     def handle_winch(self, sig, fname):
         # Ignore SIGWINCH in worker. Fixes a crash on OpenBSD.
         return
